package com.tzld.longarticle.recommend.server.service.recommend;

import cn.hutool.core.io.resource.ResourceUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.DatastatSortStrategy;
import com.tzld.longarticle.recommend.server.model.vo.IdNameVO;
import com.tzld.longarticle.recommend.server.repository.longArticle.DatastatSortStrategyRepository;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ArticlePromotionService {

    @Autowired
    DatastatSortStrategyRepository datastatSortStrategyRepository;

    private final CloseableHttpClient client = HttpPoolFactory.aigcPool();
    private static List<String> contentPoolType = Arrays.asList("autoArticlePoolLevel1", "autoArticlePoolLevel3", "autoArticlePoolLevel4");

    public void articlePromotion(String pos, String way, String accountNickName, String tag,
                                 Integer viewCountFilter, Double viewCountRateFilter, List<Integer> positionFilter) {
        String today = DateUtils.getCurrentDateStr("yyyyMMdd");

        // 获取内部表现
        List<DatastatSortStrategy> list = datastatSortStrategyRepository.getByViewCountGreaterThanEqualAndReadRateGreaterThanEqualAndPositionIn(
                viewCountFilter, viewCountRateFilter, positionFilter);
        log.info("优质{}文章数量: {}", accountNickName, list.size());

        List<DatastatSortStrategy> distinct = filterSameTitle(list);
        distinct.sort(Comparator.comparing(DatastatSortStrategy::getDateStr, Comparator.reverseOrder()));
        log.info("优质{}文章数量(去重后): {}", accountNickName, distinct.size());

        addUrlListToAccount(accountNickName, distinct, pos, way, today, tag);
    }


    private List<DatastatSortStrategy> filterSameTitle(List<DatastatSortStrategy> list) {
        List<DatastatSortStrategy> result = new ArrayList<>();
        List<String> titles = new ArrayList<>();
        for (DatastatSortStrategy datastatSortStrategy : list) {
            String title = datastatSortStrategy.getTitle();
            if (titles.contains(title)) {
                continue;
            }
            if (!TitleSimilarCheckUtil.isDuplicateContent(datastatSortStrategy.getTitle(), titles, TitleSimilarCheckUtil.ARTICLE_PROMOTION_THRESHOLD)) {
                result.add(datastatSortStrategy);
                titles.add(title);
            }
        }
        return result;
    }

    private void addUrlListToAccount(String accountNickName, List<DatastatSortStrategy> list, String pos, String way,
                                     String today, String tag) {
        List<String> urlList = list.stream().map(DatastatSortStrategy::getLink).collect(Collectors.toList());
        // 获取生产配置
        Map<String, Map<String, Map<String, String>>> produceConfig = getProduceConfig();
        if (!produceConfig.containsKey(accountNickName)) {
            log.info("account_nickname not in produceConfig: " + accountNickName);
            String planName = String.format("%d_%s_%s_%s【%s】_%s", list.size(), today, accountNickName, pos, way, today);
            createArticleUrlPlan(planName, urlList, tag);
            return;
        }

        String produceId = produceConfig.get(accountNickName).get(pos).get(way).trim();
        List<Map<String, String>> contentList = getProduceContentList(accountNickName, pos, way);

        // 获取已访问的标题和URL
        List<String> visitedTitleList = contentList.stream()
                .flatMap(content -> Arrays.stream(new String[]{content.get("referContentTitle"), content.get("title")}))
                .distinct()
                .collect(Collectors.toList());

        Set<String> visitedUrlIdList = contentList.stream()
                .map(content -> getUrlId(content.get("referContentLink")))
                .collect(Collectors.toSet());

        List<String> thisUrlIdList = urlList.stream()
                .map(this::getUrlId)
                .collect(Collectors.toList());

        // 筛选URL和标题
        List<String> filterUrlList = new ArrayList<>();
        List<String> filterTitleList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            DatastatSortStrategy item = list.get(i);
            String url = item.getLink();
            String urlId = thisUrlIdList.get(i);
            String title = item.getTitle();
            String wxSn = item.getWxSn();

            if (visitedUrlIdList.contains(urlId)) {
                continue;
            }
            if (TitleSimilarCheckUtil.isDuplicateContent(title, visitedTitleList, TitleSimilarCheckUtil.ARTICLE_PROMOTION_THRESHOLD)) {
                continue;
            }

            filterUrlList.add(url);
            filterTitleList.add(title);

            // 调用爬虫 detail 接口并保存数据
            Map<String, String> detail = getArticleDetail(url);
            if (detail != null) {
                String level = pos.equals("【1】") ? contentPoolType.get(0) : contentPoolType.get(1);
                saveArticlePoolPromotionSource(detail.get("channel_content_id"), wxSn, title, level);
            }
        }

        if (filterUrlList.isEmpty()) {
            log.info("url_list empty: " + accountNickName + ", " + pos + ", " + way);
            return;
        }

        int urlLen = filterUrlList.size();
        String planName = String.format("%d_%s_%s_%s【%s】_%s", urlLen, today, accountNickName, pos, way, today);
        log.info("url_len: " + list.size() + ", " + urlLen);

        IdNameVO<String> planInfo = createArticleUrlPlan(planName, filterUrlList, tag);
        if (produceId == null || produceId.isEmpty()) {
            log.info("{}, {}, produce plan not exist: {}, {}, {}", planInfo.getName(), planInfo.getId(), accountNickName, pos, way);
            return;
        }

        articleAddDependPlan(produceId, planInfo.getId(), planInfo.getName());
    }

    private Map<String, Map<String, Map<String, String>>> getProduceConfig() {
        // 返回生产配置的逻辑
        String json = ResourceUtil.readUtf8Str("file/ProduceConfig.json");
        return JSONObject.parseObject(json,
                new TypeReference<Map<String, Map<String, Map<String, String>>>>() {
                });
    }

    private IdNameVO<String> createArticleUrlPlan(String planName, List<String> urlList, String tag) {
        IdNameVO<String> result = new IdNameVO<>();
        if (urlList == null || urlList.isEmpty()) {
            return null;
        }

        // 从配置中读取模板文件路径
        String templateFile = ResourceUtil.readUtf8Str("file/长文_抓取计划_根据url列表抓取.json");
        if (templateFile == null) {
            return null;
        }

        // 读取模板文件内容
        JSONObject template = JSONObject.parseObject(templateFile);

        String apiUrl = template.getString("api_url");
        JSONObject data = template.getJSONObject("data");

        // 设置请求参数
        JSONObject params = data.getJSONObject("params");
        params.put("name", planName);
        if (tag != null) {
            params.put("planTag", tag);
        }
        params.put("inputModeValues", urlList);

        // 将数据转为 JSON 格式
        String requestData = data.toString();
        try {
            HttpPost httpPost = new HttpPost(apiUrl);
            StringEntity stringEntity = new StringEntity(requestData, StandardCharsets.UTF_8);
            httpPost.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpPost.setEntity(stringEntity);
            CloseableHttpResponse response = client.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    JSONObject jsonObject = JSONObject.parseObject(responseBody);
                    if (jsonObject.getInteger("code") == 0) {
                        JSONObject responseData = jsonObject.getJSONObject("data");
                        String planId = responseData.getString("id");
                        String returnedPlanName = responseData.getString("name");
                        log.info("plan_name: {}, plan_id: {}", returnedPlanName, planId);
                        result.setId(planId);
                        result.setName(returnedPlanName);
                    }
                }
            }
        } catch (Exception e) {
            log.error("createArticleUrlPlan error", e);
        }

        return result;
    }

    private List<Map<String, String>> getProduceContentList(String accountNickName, String pos, String way) {
        // 返回生产内容列表的逻辑
        return new ArrayList<>();
    }

    public String getUrlId(String url) {
        if (url == null) {
            return null;
        }
        // 检查是否包含特定的前缀
        if (url.contains("https://mp.weixin.qq.com/s/")) {
            return url.split("https://mp.weixin.qq.com/s/")[1];
        }
        Map<String, String> params = new HashMap<>();
        try {
            String pureUrl = URLDecoder.decode(url, "utf-8");
            // 解析 URL 参数
            params = parseQueryString(pureUrl);

            String biz = params.get("http://mp.weixin.qq.com/s?__biz");
            String sn = params.get("sn");
            String mid = params.get("mid");
            String idx = params.get("idx");

            if (biz != null && sn != null && mid != null && idx != null) {
                return String.format("biz=%s_mid=%s_idx=%s_sn=%s", biz, mid, idx, sn);
            }
        } catch (Exception e) {
            log.error("get_url_id error: " + url);
        }

        return params.get("sn");
    }

    // 辅助方法：解析查询参数
    private static Map<String, String> parseQueryString(String url) {
        // 示例解析逻辑
        Map<String, String> params = new java.util.HashMap<>();
        if (url.contains("?")) {
            String query = url.substring(url.indexOf("?") + 1);
            for (String param : query.split("&")) {
                String[] keyValue = param.split("=", 2);
                if (keyValue.length == 2) {
                    params.put(keyValue[0], keyValue[1]);
                }
            }
        }
        return params;
    }

    private Map<String, String> getArticleDetail(String url) {
        // 返回文章详情的逻辑
        return new HashMap<>();
    }

    private void saveArticlePoolPromotionSource(String channelContentId, String wxSn, String title, String level) {
        // 保存文章数据的逻辑
        log.info("Saved: " + channelContentId + ", " + wxSn + ", " + title + ", " + level);
    }

    private void articleAddDependPlan(String produceId, String planId, String planName) {
        // 添加依赖计划的逻辑
        log.info("Added dependency: " + produceId + " -> " + planId + " (" + planName + ")");
    }


}
